@@ -2,11 +2,11 @@ package Term::Choose::Opt::Mouse;
 
 use warnings;
 use strict;
-use 5.10.1;
+use 5.008003;
 
-our $VERSION = '1.778_04';
+our $VERSION = '1.730';
 
-use Term::Choose::Constants qw( :all );
+use Term::Choose::Constants qw( :keys :index );
 
 
 sub __mouse_info_to_key {
@@ -20,9 +20,6 @@ sub __mouse_info_to_key {
     # ..._y, ..._x: absolute position, one-based index
     my $mouse_row = $mouse_y - 1 - $self->{offset_rows};
     my $mouse_col = $mouse_x - 1;
-    if ( $self->{margin_left} ) { # ###
-        $mouse_col -= $self->{margin_left};
-    }
     if ( $mouse_row < 0 || $mouse_row > $#{$self->{rc2idx}} ) {
         return NEXT_get_key;
     }
@@ -34,7 +31,7 @@ sub __mouse_info_to_key {
         my $begin_next_col;
         if ( $self->{current_layout} == -1 ) {
             my $idx = $self->{rc2idx}[$row][$col];
-            $begin_next_col = $begin_this_col + $self->{width_elements}[$idx] + $self->{pad};
+            $begin_next_col = $begin_this_col + $self->{length}[$idx] + $self->{pad};
         }
         else {
             $begin_next_col = $begin_this_col + $self->{col_width_plus};
