@@ -2,11 +2,11 @@ package Term::Choose::Opt::SkipItems;
 
 use warnings;
 use strict;
-use 5.10.1;
+use 5.008003;
 
-our $VERSION = '1.778_04';
+our $VERSION = '1.730';
 
-use Term::Choose::Constants qw( :all );
+use Term::Choose::Constants qw( :keys :index );
 
 
 sub __key_skipped {
@@ -178,7 +178,7 @@ sub __next_valid_down {
     my $last_row = $#{$self->{rc2idx}};
     my $row = $self->{pos}[ROW];
     my $col = $self->{pos}[COL];
-    if ( $col > $self->{idx_of_last_col_in_last_row} ) {
+    if ( $self->{rest} && $col >= $self->{rest} ) {
         $last_row--;
     }
     while ( ++$row <= $last_row ) {
@@ -278,7 +278,7 @@ sub __next_valid_id_up_or_down {
     my $end = $self->{last_page_row};
     my $row = $self->{pos}[ROW];
     my $col = $self->{pos}[COL];
-    if ( $end == $#{$self->{rc2idx}} && $col > $self->{idx_of_last_col_in_last_row} ) {
+    if ( $end == $#{$self->{rc2idx}} && $self->{rest} && $col >= $self->{rest} ) {
         $end--;
     }
     my ( $row_up, $row_down ) = ( $row, $row );
